#include <iostream>
#include "i3d/image3d.h"
#include "i3d/morphology.h"
#include "threshold.h"
#include "labelling.h"
#include "rgbgray.h"

using namespace std;
using namespace i3d;


void ProcessCells(void)
{
	//opens and reads GRAY 8bit input image
	Image3d<GRAY8> img;
	img.ReadImageICS("images/cell04_3D_GRAY8.ics");

	cout << "Doing some thresholding.\n";
	Threshold(img,85);

	cout << "Doing some dilation.\n";
	Image3d<GRAY8> img_dilated;
	DilationO(img,img_dilated,2);

	//writes GRAY 8bit output image
	img_dilated.SaveImageICS("images/cell04_3D_GRAY8_thresholded.ics");

	//odstraneni malych objektu
	/*
	img=img_dilated;
	ErosionO(img,img_dilated,3);
	*/

	GRAY8 pocet_barev;
	pocet_barev=LabelImage(img_dilated);

	cout << "Obarvil jsem " << (int)pocet_barev << " bunek\n";
	img_dilated.SaveImageICS("images/cell04_3D_GRAY8_labelled.ics");
}


void ProcessBirds(void)
{
	//opens and reads 8bit RGB input image
	Image3d<RGB> img_rgb;
	img_rgb.ReadImageBMP("images/birds2_RGB8.bmp");

	cout << "Doing colour conversion.\n";
	Image3d<GRAY8> img;
	RGBtoGRAY8(img_rgb,img);
	img_rgb.DisposeData();
	img.SaveImageBMP("images/birds2_GRAY8.bmp");

	cout << "Doing some thresholding.\n";
	Threshold(img,200);

	//musi se "otocit" vysledek prahovani!
	GRAY8* p=img.GetFirstVoxelAddr();
	for (size_t i=0; i < img.GetImageSize(); ++i, ++p) *p ^= 1;

	cout << "Doing some dilation.\n";
	Image3d<GRAY8> img_dilated;
	DilationO(img,img_dilated,2);

	//writes GRAY 8bit output image
	img_dilated.SaveImageBMP("images/birds2_GRAY8_thresholded.bmp");

	//odstraneni malych objektu
	img=img_dilated;
	OpeningO(img,img_dilated,3);

	GRAY8 pocet_barev;
	pocet_barev=LabelImage(img_dilated);

	cout << "Obarvil jsem " << (int)pocet_barev << " ptaku\n";
	img_dilated.SaveImageBMP("images/birds2_GRAY8_labelled.bmp");
}


int main(void) {
	try {
		//chytani vyjimek, umoznuje zobrazit chybova hlaseni knihoven i3dlibs
		//tato konstrukce rika, ze v tomto bloku {...} se maji chytat vyjimky

		//ProcessCells();
		ProcessBirds();

		//pokud se program dostal az sem, nenastal zadny problem
		return(0);

	} //konec bloku "try" - chytani ruznych vyjimek
	catch (const char *e)
	{
		cout << e << endl;
	}
	catch (std::string &e)
	{
		cout << e << endl;
	}
	catch (i3d::IOException& e)
	{
		cout << e << endl;
	}
	catch (i3d::InternalException& e)
	{
		cout << e << endl;
	}
	catch (std::bad_alloc&)
	{
		cout << "Not enough memory." << endl;
	}
	catch (...)
	{
		cout << "System exception." << endl;
	}

	//pokud se program dostal sem, nastal predtim nejaky problem
	return(1);
}
